@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_arch_c =
- *   "$Id: arch.c,v 1.17 2001/12/03 07:51:39 mwedel Exp $";
+ *   "$Id: arch.c,v 1.23 2002/07/15 04:57:11 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 #include <global.h>
@@ -71,24 +71,27 @@ archetype *find_archetype_by_object_name(char *name) {
  * - name: The name we're searching for (ex: "writing pen");
  * Return value:
  * - a corresponding object if found; a singularity object if not found.
+ * Note by MSW - it appears that it takes the full name and keeps
+ * shortening it until it finds a match.  I re-wrote this so that it
+ * doesn't malloc it each time - not that this function is used much,
+ * but it otherwise had a big memory leak.
  */
 object *get_archetype_by_object_name(char *name) {
-  archetype *at;
-  char *tmpname;
-  int i;
-  for(i=strlen(name); i>0;i--)
-  {
-        tmpname = (char *)(malloc(i+1));
-        strncpy(tmpname,name,i);
-        tmpname[i] = 0x0;
+    archetype *at;
+    char tmpname[MAX_BUF];
+    int i;
+
+    strncpy(tmpname,name,MAX_BUF-1);
+    tmpname[MAX_BUF-1] = 0;
+    for(i=strlen(tmpname); i>0; i--) {
+	tmpname[i] = 0;
         at = find_archetype_by_object_name(tmpname);
         if (at !=NULL)
         {
-                free(tmpname);
-                return arch_to_object(at);
-        };
-  };
-  return create_singularity(name);
+	    return arch_to_object(at);
+        }
+    }
+    return create_singularity(name);
 }
 
  /* GROS - find_best_weapon_used_match and item_matched_string moved there */
@@ -130,75 +133,69 @@ object *get_archetype_by_object_name(char *name) {
   */
 int item_matched_string(object *pl, object *op, char *name)
 {
-   char *cp, local_name[MAX_BUF];
-   int count,retval=0;
-   strcpy(local_name, name);	/* strtok is destructive to name */
-   for (cp=strtok(local_name,","); cp; cp=strtok(NULL,",")) {
-     while (cp[0]==' ') ++cp;	/* get rid of spaces */
-     /*	LOG(llevDebug,"Trying to match %s\n", cp);*/
-     /* All is a very generic match - low match value */
-     if (!strcmp(cp,"all")) return 1;
-     /* unpaid is a little more specific */
-     if (!strcmp(cp,"unpaid") && QUERY_FLAG(op,FLAG_UNPAID)) return 2;
-     if (!strcmp(cp,"cursed") && QUERY_FLAG(op,FLAG_KNOWN_CURSED) &&
-       (QUERY_FLAG(op,FLAG_CURSED) ||QUERY_FLAG(op,FLAG_DAMNED)))
-       return 2;
-     /* Allow for things like '100 arrows' */
-     if ((count=atoi(cp))!=0) {
-       cp=strchr(cp, ' ');
-       while (cp && cp[0]==' ') ++cp;	/* get rid of spaces */
-     }
-     else
-     {
-       if (pl->type==PLAYER)
-       {
-         count=pl->contr->count;
-       }
-       else
-       {
-         count = 0;
-       };
-     };
-     if (!cp || cp[0]=='\0' || count<0) return 0;
-     /* base name matched - not bad */
-     if (strcasecmp(cp,op->name)==0 && !count) return 4;
-     else if (count>1) {	/* Need to plurify name for proper match */
-     char newname[MAX_BUF];
-     strcpy(newname, op->name);
-     if (QUERY_FLAG(op,FLAG_NEED_IE)) {
-       char *cp1=strrchr(newname,'y');
-       if(cp1!=NULL)
-         *cp1='\0'; /* Strip the 'y' */
-       strcat(newname,"ies");
-     }
-     else strcat(newname,"s");
-     if (!strcasecmp(newname,cp)) {
-       pl->contr->count=count;	/* May not do anything */
-       return 6;
-     }
-   }
-   else if (count==1) {
-     if (!strcasecmp(op->name,cp)) {
-       pl->contr->count=count;	/* May not do anything */
-       return 6;
-     }
-   }
-   if (!strcasecmp(cp,query_name(op))) retval=20;
-   else if (!strcasecmp(cp,query_short_name(op))) retval=18;
-   else if (!strcasecmp(cp,query_base_name(op,0))) retval=16;
-   else if (!strcasecmp(cp,query_base_name(op,1))) retval=16;
-   else if (!strncasecmp(cp,query_base_name(op,0),
-     MIN(strlen(cp),strlen(query_base_name(op,0))))) retval=14;
-   else if (!strncasecmp(cp,query_base_name(op,1),
-     MIN(strlen(cp),strlen(query_base_name(op,1))))) retval=14;
-   if (retval) {
-     if (pl->type == PLAYER)
-     {
-       pl->contr->count=count;
-     };
-     return retval;
-   }
-   }
+    char *cp, local_name[MAX_BUF];
+    int count,retval=0;
+    strcpy(local_name, name);	/* strtok is destructive to name */
+
+    for (cp=strtok(local_name,","); cp; cp=strtok(NULL,",")) {
+	while (cp[0]==' ') ++cp;	/* get rid of spaces */
+
+	/*	LOG(llevDebug,"Trying to match %s\n", cp);*/
+	/* All is a very generic match - low match value */
+	if (!strcmp(cp,"all")) return 1;
+
+	/* unpaid is a little more specific */
+	if (!strcmp(cp,"unpaid") && QUERY_FLAG(op,FLAG_UNPAID)) return 2;
+	if (!strcmp(cp,"cursed") && QUERY_FLAG(op,FLAG_KNOWN_CURSED) &&
+	    (QUERY_FLAG(op,FLAG_CURSED) ||QUERY_FLAG(op,FLAG_DAMNED)))
+	    return 2;
+
+	if (!strcmp(cp,"unlocked") && !QUERY_FLAG(op, FLAG_INV_LOCKED))
+	    return 2;
+
+	/* Allow for things like '100 arrows' */
+	if ((count=atoi(cp))!=0) {
+	    cp=strchr(cp, ' ');
+	    while (cp && cp[0]==' ') ++cp;	/* get rid of spaces */
+	}
+	else {
+	    if (pl->type==PLAYER)
+		count=pl->contr->count;
+	    else
+		count = 0;
+	}
+
+	if (!cp || cp[0]=='\0' || count<0) return 0;
+
+	/* base name matched - not bad */
+	if (strcasecmp(cp,op->name)==0 && !count) return 4;
+
+	else if (count>1) {	/* Need to plurify name for proper match */
+	    if (!strcasecmp(cp,op->name_pl)) {
+		pl->contr->count=count;	/* May not do anything */
+		return 6;
+	    }
+	}
+	else if (count==1) {
+	    if (!strcasecmp(op->name,cp)) {
+		pl->contr->count=count;	/* May not do anything */
+		return 6;
+	    }
+	}
+	if (!strcasecmp(cp,query_name(op))) retval=20;
+	else if (!strcasecmp(cp,query_short_name(op))) retval=18;
+	else if (!strcasecmp(cp,query_base_name(op,0))) retval=16;
+	else if (!strcasecmp(cp,query_base_name(op,1))) retval=16;
+	else if (!strncasecmp(cp,query_base_name(op,0),
+			      MIN(strlen(cp),strlen(query_base_name(op,0))))) retval=14;
+	else if (!strncasecmp(cp,query_base_name(op,1),
+			      MIN(strlen(cp),strlen(query_base_name(op,1))))) retval=14;
+	if (retval) {
+	    if (pl->type == PLAYER)
+		pl->contr->count=count;
+	    return retval;
+	}
+    }
    return 0;
 }
 
@@ -280,6 +277,7 @@ void free_all_archs()
 	else next=at->next;
 	if (at->name) free_string(at->name);
 	if (at->clone.name) free_string(at->clone.name);
+	if (at->clone.name_pl) free_string(at->clone.name_pl);
 	if (at->clone.title) free_string(at->clone.title);
 	if (at->clone.race) free_string(at->clone.race);
 	if (at->clone.slaying) free_string(at->clone.slaying);
@@ -304,6 +302,7 @@ archetype *get_archetype_struct() {
   new->name=NULL;
   new->clone.other_arch=NULL;
   new->clone.name=NULL;
+  new->clone.name_pl=NULL;
   new->clone.title=NULL;
   new->clone.race=NULL;
   new->clone.slaying=NULL;
@@ -321,63 +320,72 @@ archetype *get_archetype_struct() {
  * of archetype-structures.
  */
 void first_arch_pass(FILE *fp) {
-  object *op;
-  archetype *at,*prev=NULL,*last_more=NULL;
-  int i,first=2;
-  int xo,yo;
+    object *op;
+    archetype *at,*head=NULL,*last_more=NULL;
+    int i,first=2;
 
-  op=get_object();
-  op->arch=first_archetype=at=get_archetype_struct();
-  while((i=load_object(fp,op,first,0))) {
-    first=0;
-    copy_object(op,&at->clone);
-    at->clone.speed_left= (float) (-0.1);
-    switch(i) {
-    case LL_NORMAL: /* A new archetype, just link it with the previous */
-      if(last_more!=NULL)
-        last_more->next=at;
-      if(prev!=NULL)
-        prev->next=at;
-      prev=last_more=at;
+    op=get_object();
+    op->arch=first_archetype=at=get_archetype_struct();
+
+    while((i=load_object(fp,op,first,0))) {
+	first=0;
+	copy_object(op,&at->clone);
+	at->clone.speed_left= (float) (-0.1);
+	/* copy the body_info to the body_used - this is only really
+	 * need for monsters, but doesn't hurt to do it for everything.
+	 * by doing so, when a monster is created, it has good starting
+	 * values for the body_used info, so when items are created 
+	 * for it, they can be properly equipped.
+	 */
+	memcpy(&at->clone.body_used, &op->body_info, sizeof(op->body_info));
+
+	switch(i) {
+	    case LL_NORMAL: /* A new archetype, just link it with the previous */
+		if(last_more!=NULL)
+		    last_more->next=at;
+		if(head!=NULL)
+		    head->next=at;
+		head=last_more=at;
 #if 0
-      if(!op->type)
-          LOG(llevDebug," WARNING: Archetype %s has no type info!\n", op->arch->name);
+		if(!op->type)
+		    LOG(llevDebug," WARNING: Archetype %s has no type info!\n", op->arch->name);
 #endif
-      
-      op->quick_pos = 0; /* assume as base a single arch */
-      at->clone.quick_pos = 0; /* sic */
-      break;
-#if 0
-    case 2:
-      LOG(llevError,"Error: Archetype with inventory is illegal!\n");
-      exit(-1);
-#endif
-    case LL_MORE: /* Another part of the previous archetype, link it correctly */
-      at->head=prev;
-      at->clone.head = &prev->clone;
-      if(last_more!=NULL) {
-          last_more->more=at;
-          last_more->clone.more = &at->clone;
-      }
-      last_more=at;
-  
-      xo = at->clone.x;
-      if(xo <0)
-          xo=(xo*-1)|8;
-      yo = at->clone.y;
-      if(yo <0)
-          yo=(yo*-1)|8;
-      at->clone.quick_pos = 0|(xo<<4)|yo;
-      prev->clone.quick_pos = 255; /* mark head tile */
-      
-      break;
+		at->tail_x = 0;
+		at->tail_y = 0;
+		break;
+
+	    case LL_MORE: /* Another part of the previous archetype, link it correctly */
+
+		at->head=head;
+		at->clone.head = &head->clone;
+		if(last_more!=NULL) {
+		    last_more->more=at;
+		    last_more->clone.more = &at->clone;
+		}
+		last_more=at;
+
+		/* If this multipart image is still composed of individual small
+		 * images, don't set the tail_.. values.  We can't use them anyways,
+		 * and setting these to zero makes the map sending to the client much
+		 * easier as just looking at the head, we know what to do.
+		 */
+		if (at->clone.face != head->clone.face) {
+		    head->tail_x = 0;
+		    head->tail_y = 0;
+		} else {
+		    if (at->clone.x > head->tail_x) head->tail_x = at->clone.x;
+		    if (at->clone.y > head->tail_y) head->tail_y = at->clone.y;
+		}
+		break;
+
+	}
+
+	at=get_archetype_struct();
+	clear_object(op);
+	op->arch=at;
     }
-    at=get_archetype_struct();
-    clear_object(op);
-    op->arch=at;
-  }
-  free(at);
-  free_object(op);
+    free(at);
+    free_object(op);
 }
 
 /*
@@ -522,6 +530,7 @@ object *create_singularity(char *name) {
   sprintf(buf,"singluarity (%s)",name);
   op = get_object();
   op->name = add_string(buf);
+  op->name_pl = add_string(buf);
   SET_FLAG(op,FLAG_NO_PICK);
   return op;
 }
