@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_button_c =
- *   "$Id: button.c,v 1.15 2001/07/14 04:04:53 mwedel Exp $";
+ *   "$Id: button.c,v 1.18 2002/09/05 06:19:27 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 #include <global.h>
@@ -541,16 +541,18 @@ int get_button_value(object *button) {
  */
  
 void do_mood_floor(object *op, object *op2) {
-        object *tmp;
-        object *tmp2;
+    object *tmp;
+    object *tmp2;
 
-        for(tmp=get_map_ob(op->map,op->x,op->y);
-                tmp&&!QUERY_FLAG(tmp,FLAG_MONSTER);tmp=tmp->above)  
-        	if(tmp->above==NULL) break;
+    for (tmp = GET_MAP_OB(op->map, op->x, op->y); tmp; tmp=tmp->above)
+	if (QUERY_FLAG(tmp, FLAG_MONSTER)) break;
 
-        if(tmp->type==PLAYER) return;
+    /* doesn't effect players, and if there is a player on this space, won't also
+     * be a monster here.
+     */
+    if (!tmp || tmp->type == PLAYER) return;
 
-	switch(op->last_sp) { 
+    switch(op->last_sp) { 
 	  case 0:			/* furious--make all monsters mad */ 
 		if(QUERY_FLAG(tmp, FLAG_UNAGGRESSIVE))
 			CLEAR_FLAG(tmp, FLAG_UNAGGRESSIVE);
@@ -601,8 +603,7 @@ void do_mood_floor(object *op, object *op2) {
 	  default:
 		break;
 
-	}
-        return;
+    }
 }
 
 /* this function returns the object it matches, or NULL if non.
