@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_porting_c =
- *   "$Id: porting.c,v 1.12 2001/10/30 02:30:19 michtoen Exp $";
+ *   "$Id: porting.c,v 1.14 2002/09/07 05:55:18 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 /* This file contains various functions that are not really unique for
@@ -509,7 +509,8 @@ FILE *open_and_uncompress(char *name,int flag, int *compressed) {
       return fp;
     }
   }
-  LOG(llevDebug, "Can't open %s\n", name);
+  /* Let the caller print an error if it wants to */
+/*  LOG(llevDebug, "Can't open %s\n", name);*/
   return NULL;
 }
 
