@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_config_h =
- *   "$Id: config.h,v 1.26 2001/12/23 08:17:36 mwedel Exp $";
+ *   "$Id: config.h,v 1.30 2002/09/11 06:21:46 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 /* This file contains various #defines that select various options.
@@ -269,17 +269,6 @@
 
 #define MAX_TIME	120000
 
-/* NEW_WORLD_MAP
- * If you have checked out the maps-bigworld directory from CVS,
- * enable this define below.  The only thing it currently does is
- * update the EMERGENCY_... positions to fall properly into scorn on 
- * the new map.
- */
-
-/*
-#define NEW_WORLD_MAP
-*/
-
 /* This is a modification to the cone spells.  If a cone spell hits
  * a monster, the spell stops progressing (same is true for parties
  * of players).  This weakens the cones spells a bit.  If it is not
@@ -406,17 +395,15 @@
  * Set to true if you want to use the new (0.95.3) experience system.
  * The system has the following changes:
  *
- * 1) 2000 exp need for second level.  All additional levels are shifted
- *  accordingly.
- * 2) Your stats in your skill categories don't affect the amount of
+ * 1) Your stats in your skill categories don't affect the amount of
  *  exp you get.
- * 3) Your level difference doesn't affect the exp you get for killing
+ * 2) Your level difference doesn't affect the exp you get for killing
  *  monsters.
  *
  * If this is set to FALSE out, you get the old system.
  *
  * This can also be set by using -simple-exp/+simple-exp as a run
- * time option.
+ * time option, or update the settings file
  */
 
 #define SIMPLE_EXP TRUE
@@ -767,7 +754,7 @@
  * If you want to have a Message Of The Day file, define MOTD to be
  * the file with the message.  If the file doesn't exist or if it
  * is empty, no message will be displayed.
- * (It resides in the LIBDIR directory)
+ * (It resides in the CONFDIR directory)
  */
 
 #ifndef MOTD
@@ -776,7 +763,7 @@
 
 /*
  * You can restrict playing in certain times by creating a PERMIT_FILE
- * in LIBDIR. See teh sample for usabe notes.
+ * in CONFDIR. See the sample for usage notes.
  */
 
 #define PERM_FILE "forbid"
@@ -785,7 +772,7 @@
  * If you want to take the game down while installing new versions, or
  * for other reasons, put a message into the SHUTDOWN_FILE file.
  * Remember to delete it when you open the game again.
- * (It resides in the LIBDIR directory)
+ * (It resides in the CONFDIR directory)
  */
 
 #ifndef SHUTDOWN_FILE
@@ -854,18 +841,12 @@
 
 #ifdef WIN32 /* change define path city */
 #  define EMERGENCY_MAPPATH "\\city\\city"
-#  define EMERGENCY_X 5
-#  define EMERGENCY_Y 37
+#  define EMERGENCY_X 15
+#  define EMERGENCY_Y 19
 #else
-#  ifdef NEW_WORLD_MAP
-#    define EMERGENCY_MAPPATH "/world/world_105_115"
-#    define EMERGENCY_X 5
-#    define EMERGENCY_Y 37
-#  else
-#    define EMERGENCY_MAPPATH "/city/city"
-#    define EMERGENCY_X 15
-#    define EMERGENCY_Y 19
-#  endif
+#  define EMERGENCY_MAPPATH "/city/city"
+#  define EMERGENCY_X 15
+#  define EMERGENCY_Y 19
 #endif
 
 
