@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_global_h =
- *   "$Id: global.h,v 1.25 2002/01/02 00:26:28 garbled Exp $";
+ *   "$Id: global.h,v 1.30 2002/09/07 22:08:07 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
 #ifndef GLOBAL_H
@@ -59,7 +59,6 @@ typedef unsigned char	uint8;
 typedef signed char	sint8;
 typedef unsigned short Fontindex;
 
-#define DELETE_STRING(__str_)  free_string(__str_);__str_=NULL;
 
 
 /* global stuff used by new skill/experience system -b.t.
@@ -188,7 +187,10 @@ EXTERN archetype *ring_arch,*amulet_arch,*staff_arch,*crown_arch;
 EXTERN char *undead_name;	/* Used in hit_player() in main.c */
 
 EXTERN Animations *animations;
-EXTERN int num_animations,animations_allocated;
+EXTERN int num_animations,animations_allocated, bmaps_checksum;
+
+/* Rotate right from bsd sum. This is used in various places for checksumming */
+#define ROTATE_RIGHT(c) if ((c) & 01) (c) = ((c) >>1) + 0x80000000; else (c) >>= 1;
 
 
 #define SET_ANIMATION(ob,newanim) ob->face=&new_faces[animations[ob->animation_id].faces[newanim]]
@@ -230,6 +232,14 @@ EXTERN char *font_graphic;
  * makes the code a bit cleaner when doing so.
  */
 #define FREE_AND_CLEAR(xyz) {free(xyz); xyz=NULL; }
+#define FREE_AND_CLEAR_STR(xyz) {free_string(xyz); xyz=NULL; }
+
+/* FREE_AND_COPY is for the shared string - it is handy enough
+ * to use all over the place.
+ */
+#define FREE_AND_COPY(sv,nv) { if (sv) free_string(sv); sv=add_string(nv); }
+
+#define DELETE_STRING(__str_)  free_string(__str_);__str_=NULL;
 
 #ifdef CALLOC
 #undef CALLOC
@@ -271,6 +281,7 @@ typedef struct Settings {
     uint8   daemonmode;     /* If true, detach and become daemon */
     int	    argc;	    /* parameters that were passed to the program */
     char    **argv;	    /* Only used by xio.c, so will go away at some time */
+    char    *confdir;	    /* configuration files */
     char    *datadir;	    /* read only data files */
     char    *localdir;	    /* read/write data files */
     char    *playerdir;	    /* Where the player files are */
@@ -293,6 +304,7 @@ typedef struct Settings {
     char    meta_host[MAX_BUF];	    /* Hostname of this host */
     uint16  meta_port;		    /* Port number to use for updates */
     char    meta_comment[MAX_BUF];  /* Comment we send to the metaserver */
+
     uint32  worldmapstartx;	    /* starting x tile for the worldmap */
     uint32  worldmapstarty;	    /* starting y tile for the worldmap */
     uint32  worldmaptilesx;	    /* number of tiles wide the worldmap is */
@@ -300,6 +312,9 @@ typedef struct Settings {
     uint32  worldmaptilesizex;	    /* number of squares wide in a wm tile */
     uint32  worldmaptilesizey;	    /* number of squares high in a wm tile */
     uint16  dynamiclevel;	    /* how dynamic is the world? */
+
+    char    *emergency_mapname;	    /* map to return players to in emergency */
+    uint16  emergency_x, emergency_y;	/* coordinates to use on that map */
 } Settings;
 
 extern Settings settings;
