@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_includes_h =
- *   "$Id: includes.h,v 1.9 2001/10/14 07:57:14 gros Exp $";
+ *   "$Id: includes.h,v 1.10 2002/07/15 04:57:12 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 1994 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mark@pyramid.com
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 #ifndef INCLUDES_H
@@ -110,5 +110,11 @@
 #define FALSE 0
 #endif
 
+#if defined(vax) || defined(ibm032)
+size_t strftime(char *, size_t, const char *, const struct tm *);
+time_t mktime(struct tm *);
+#endif
+
+
 #endif /* INCLUDES_H */
 
