@@ -50,6 +50,8 @@ extern void verify_button_links(mapstruct *map);
 /* exp.c */
 extern int new_exp(object *ob);
 extern int has_ability(object *ob);
+extern void init_experience(void);
+extern void dump_experience(void);
 /* friend.c */
 extern void add_friendly_object(object *op);
 extern void remove_friendly_object(object *op);
@@ -122,6 +124,8 @@ extern void write_todclock(void);
 extern void init_clocks(void);
 extern void init_attackmess(void);
 /* item.c */
+extern int get_power_from_ench(int ench);
+extern int calc_item_power(object *op, int flag);
 extern char *describe_resistance(object *op, int newline);
 extern char *query_weight(object *op);
 extern char *get_levelnumber(int i);
@@ -130,7 +134,7 @@ extern char *ring_desc(object *op);
 extern char *query_short_name(object *op);
 extern char *query_name(object *op);
 extern char *query_base_name(object *op, int plural);
-extern char *describe_item(object *op);
+extern char *describe_item(object *op, object *owner);
 extern int is_magical(object *op);
 extern int need_identify(object *op);
 extern int look_up_spell_name(char *spname);
@@ -162,17 +166,6 @@ extern void calc_perm_exp(object *op);
 extern int adjust_exp(object *op, int exp);
 extern int check_dm_add_exp_to_obj(object *exp_ob, int i);
 extern void apply_death_exp_penalty(object *op);
-/* loader.c */
-extern int lex_load(object *op, int map_flags);
-extern void yyrestart(FILE *input_file);
-extern void yy_load_buffer_state(void);
-extern int yyerror(char *s);
-extern int load_object(FILE *fp, object *op, int bufstate, int map_flags);
-extern int set_variable(object *op, char *buf);
-extern void save_double(char *buf, char *name, double v);
-extern void init_vars(void);
-extern char *get_ob_diff(object *op, object *op2);
-extern void save_object(FILE *fp, object *op, int flag);
 /* logger.c */
 extern void LOG(LogLevel logLevel, char *format, ...);
 /* los.c */
@@ -202,7 +195,6 @@ extern int blocks_magic(mapstruct *m, int x, int y);
 extern int blocks_cleric(mapstruct *m, int x, int y);
 extern int blocked(mapstruct *m, int x, int y);
 extern int blocked_link(object *ob, int x, int y);
-extern int blocked_two(object *op, int x, int y);
 extern int arch_blocked(archetype *at, mapstruct *m, int x, int y);
 extern int arch_out_of_map(archetype *at, mapstruct *m, int x, int y);
 extern void fix_container(object *container);
@@ -252,6 +244,7 @@ extern void expand_objects(void);
 extern object *get_object(void);
 extern void update_turn_face(object *op);
 extern void update_ob_speed(object *op);
+extern void remove_from_active_list(object *op);
 extern void update_object(object *op, int action);
 extern void free_object(object *ob);
 extern int count_free(void);
@@ -274,6 +267,7 @@ extern object *present_arch_in_ob(archetype *at, object *op);
 extern void set_cheat(object *op);
 extern int find_free_spot(archetype *at, mapstruct *m, int x, int y, int start, int stop);
 extern int find_first_free_spot(archetype *at, mapstruct *m, int x, int y);
+extern void get_search_arr(int *search_arr);
 extern int find_dir(mapstruct *m, int x, int y, object *exclude);
 extern int distance(object *ob1, object *ob2);
 extern int find_dir_2(int x, int y);
@@ -296,9 +290,7 @@ extern FILE *open_and_uncompress(char *name, int flag, int *compressed);
 extern void close_and_delete(FILE *fp, int compressed);
 extern void make_path_to_file(char *filename);
 /* player.c */
-extern player *get_player_ob(void);
 extern void free_player(player *pl);
-extern void generate_ext_title(player *pl);
 extern object *find_skill(object *op, int skillnr);
 extern int atnr_is_dragon_enabled(int attacknr);
 extern int is_dragon_pl(object *op);
@@ -390,3 +382,14 @@ extern int random_roll(int min, int max, object *op, int goodbad);
 extern int die_roll(int num, int size, object *op, int goodbad);
 extern int rndm(int min, int max);
 extern void decay_objects(mapstruct *m);
+/* loader.c */
+extern int lex_load(object *op, int map_flags);
+extern void yyrestart(FILE *input_file);
+extern void yy_load_buffer_state(void);
+extern int yyerror(char *s);
+extern int load_object(FILE *fp, object *op, int bufstate, int map_flags);
+extern int set_variable(object *op, char *buf);
+extern void save_double(char *buf, char *name, double v);
+extern void init_vars(void);
+extern char *get_ob_diff(object *op, object *op2);
+extern void save_object(FILE *fp, object *op, int flag);
