@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_logger_h =
- *   "$Id: logger.h,v 1.4 1997/01/05 09:59:01 master Exp $";
+ *   "$Id: logger.h,v 1.3 2000/05/21 21:41:45 jec Exp $";
  */
 
 /*
@@ -29,7 +29,7 @@
 #define LOGGER_H
 
 typedef enum LogLevel {
-  llevError = 0, llevDebug = 1, llevMonster = 2
+  llevError = 0, llevInfo = 1, llevDebug = 2, llevMonster = 3
 } LogLevel;
 
 #endif /* LOGGER_H */
