@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: map.h,v 1.15 2002/01/01 23:37:54 garbled Exp $";
+ *   "$Id: map.h,v 1.18 2002/07/15 04:25:40 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 /*
@@ -34,6 +34,8 @@
 #ifndef MAP_H
 #define MAP_H
 
+#define MAP_LAYERS		4
+
 /* This is when the map will reset */
 #define MAP_WHEN_RESET(m)	((m)->reset_time)
 
@@ -128,8 +130,8 @@
 
 typedef struct MapSpace {
     object	*bottom;	/* lowest object on this space */
-    New_Face	*faces[3];	/* faces for the 3 layers */
-    object	*faces_obj[3];	/* face objects for the 3 layers */
+    New_Face	*faces[MAP_LAYERS];	/* faces for the 3 layers */
+    object	*faces_obj[MAP_LAYERS];	/* face objects for the 3 layers */
     uint8	flags;		/* flags about this space (see the P_ values above) */
     sint8	light;		/* How much light this space provides */
 } MapSpace;
