@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_newserver_h =
- *   "$Id: newserver.h,v 1.21 2001/12/31 08:35:48 mwedel Exp $";
+ *   "$Id: newserver.h,v 1.26 2002/07/15 04:57:12 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -36,18 +36,30 @@
 
 #ifndef NEWSERVER_H
 #define NEWSERVER_H
-#define MAXMAPCELLFACES 50
+
+/* Reduce this from 50 to 5 - as it is now, there will never be more
+ * than 3 anyways.
+ */
 
 #define NUM_LOOK_OBJECTS 50
 
 struct MapCell {
-  short faces[MAXMAPCELLFACES];
-  char quick_pos[MAXMAPCELLFACES];
-  int count;
+  short faces[MAP_LAYERS];
+  int count;	/* This is really darkness in the map1 command */
 };
 
+/* This basically defines the largest size an 
+ * archetype may be - it is used for allocation of
+ * some structures, as well as determining how far
+ * we should look for the heads of big images.
+ */
+#define MAX_HEAD_OFFSET	    6
+
+#define MAX_CLIENT_X (MAP_CLIENT_X + MAX_HEAD_OFFSET)
+#define MAX_CLIENT_Y (MAP_CLIENT_Y + MAX_HEAD_OFFSET)
+
 struct Map {
-  struct MapCell cells[MAP_CLIENT_X][MAP_CLIENT_Y];
+  struct MapCell cells[MAX_CLIENT_X][MAX_CLIENT_Y];
 };
 
 /* True max is 16383 given current map compaction method */
@@ -55,7 +67,7 @@ struct Map {
 #define MAXANIMNUM  2000
 
 struct statsinfo {
-    char *range, *title, *ext_title;
+    char *range, *title;
 };
 
 
@@ -70,6 +82,12 @@ enum Sock_Status {Ns_Avail, Ns_Add, Ns_Dead, Ns_Old};
 /* Reserver 0 for neither of these being set */
 enum Old_Mode {Old_Listen=1, Old_Player=2};
 
+/* Only one map mode can actually be used, so lets make it a switch
+ * instead of having a bunch of different fields that needed to
+ * get toggled.
+ */
+enum MapMode {Map0Cmd = 0, Map1Cmd = 1, Map1aCmd=2 };
+
 /* The following is the setup for a ring buffer for storing outbut
  * data that the OS can't handle right away.
  */
@@ -98,8 +116,7 @@ typedef struct NewSocket {
     uint32  ext_title_flag;  /* if 1, we should generate and send a new ext_title update */
     uint32  sound:1;	    /* does the client want sound */
     uint32  skillexp:1;	    /* does the client want skill exp data - MT*/
-    uint32  map1cmd:1;	    /* Always use map1 protocol command */
-    uint32  map2cmd:1;	    /* Always use map2 protocol command */
+    enum MapMode mapmode;   /* Type of map commands the client wants. */
     uint32  newmapcmd:1;    /* Send newmap command when entering new map SMACFIGGEN*/
     uint32  darkness:1;	    /* True if client wants darkness information */
     uint32  image2:1;	    /* Client wants image2/face2 commands */
@@ -109,6 +126,7 @@ typedef struct NewSocket {
     uint32  can_write:1;    /* Can we write to this socket? */
     uint16  look_position;  /* start of drawing of look window */
     uint8   mapx, mapy;	    /* How large a map the client wants */
+    uint8   itemcmd;	    /* What version of the 'item' protocol command to use */
 
     /* Below here is information only relevant for old sockets */
     char    *comment;	    /* name or listen comment */
@@ -131,7 +149,7 @@ typedef struct Socket_Info {
 extern Socket_Info socket_info;
 
 #define VERSION_CS 1023    /* version >= 1023 understand setup cmd */
-#define VERSION_SC 1026
+#define VERSION_SC 1027
 #define VERSION_INFO "Crossfire Server"
 
 
