@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_version_h =
- *   "$Id: version.h,v 1.24 2002/03/20 06:27:46 mwedel Exp $";
+ *   "$Id: version.h,v 1.27 2002/09/07 22:08:07 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,11 +23,14 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
+/* This file is not needed any more. Version is set by autoconf/automake */
+#if 0
 #ifdef MAKE
-#define Version "1.1.0"
+#define Version "1.3.0"
 #else
-#define VERSION "1.1.0"
+#define VERSION "1.3.0"
+#endif
 #endif
