@@ -1,11 +1,12 @@
 /*
  * static char *rcsid_alchemy_c =
- *   "$Id: alchemy.c,v 1.10 2001/10/15 21:12:08 garbled Exp $";
+ *   "$Id: alchemy.c,v 1.12 2002/07/15 04:25:40 mwedel Exp $";
  */
+
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -22,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 /* March 96 - Laid down original code. -b.t. thomas@astro.psu.edu */
@@ -399,6 +400,8 @@ void alchemy_failure_effect(object *op,object *cauldron,recipe *rp,int danger) {
 	tmp->material=material;
 	free_string(tmp->name);
 	tmp->name=add_string("slag");
+	if (tmp->name_pl) free_string(tmp->name_pl);
+	tmp->name_pl=add_string("slags");
         item=insert_ob_in_ob(tmp,cauldron);
 	CLEAR_FLAG(tmp,FLAG_CAN_ROLL);
 	CLEAR_FLAG(tmp,FLAG_NO_PICK);
