@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_ban_c =
- *   "$Id: ban.c,v 1.5 2002/01/02 00:26:28 garbled Exp $";
+ *   "$Id: ban.c,v 1.6 2002/09/07 22:08:07 mwedel Exp $";
  */
 
 /*
@@ -34,7 +34,7 @@ int checkbanned(char *login, char *host)
   int           num1;
   int   Hits=0;                 /* Hits==2 means we're banned */
 
-  sprintf (buf, "%s/%s", settings.datadir, BANFILE);
+  sprintf (buf, "%s/%s", settings.confdir, BANFILE);
   if ((bannedfile = fopen(buf, "r")) == NULL) {
     LOG (llevDebug, "Could not find file Banned file.\n");
     return(0);
