@@ -1,11 +1,37 @@
 /*
- * This file contains various commands that are used by the new client/server
- *	++Jam
+ * static char *rcsid_c_new_c =
+ *   "$Id: c_new.c,v 1.6 2002/03/20 06:27:46 mwedel Exp $";
  */
 
+/*
+    CrossFire, A Multiplayer game for X-windows
+
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
+    Copyright (C) 1992 Frank Tore Johansen
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+
+    The author can be reached via e-mail to crossfire-devel@real-time.com
+*/
+
+/* This file deals with administrative commands from the client. */
 #include <global.h>
 #include <commands.h>
+#ifndef __CEXTRACT__
 #include <sproto.h>
+#endif
 
 #ifndef tolower
 #define tolower(C)      (((C) >= 'A' && (C) <= 'Z')? (C) - 'A' + 'a': (C))
@@ -45,21 +71,25 @@ int execute_newserver_command(object *pl, char *command)
     pl->contr->has_hit=0;
 
     cp=strchr(command, ' ');
-    if (cp) { 
+    if (cp) {
 	*(cp++) ='\0';
 	while (*cp==' ') cp++;
     }
 
+    csp = find_plugin_command(command,pl);
+
+    if (!csp)
     csp = find_command_element(command, NewServerCommands,
 		NewServerCommandSize);
     if (!csp)
 	csp = find_command_element(command, Commands, CommandsSize);
-
+    if (!csp)
+        csp = find_command_element(command, CommunicationCommands, CommunicationCommandSize);
     if (!csp && QUERY_FLAG(pl, FLAG_WIZ))
 	csp = find_command_element(command, WizCommands, WizCommandsSize);
 
     if (csp==NULL) {
-	    new_draw_info_format(NDI_UNIQUE, 0,pl, 
+	    new_draw_info_format(NDI_UNIQUE, 0,pl,
 		"'%s' is not a valid command.", command);
 	    return 0;
     }
