@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_object_c =
- *   "$Id: c_object.c,v 1.34 2002/03/20 06:27:46 mwedel Exp $";
+ *   "$Id: c_object.c,v 1.38 2002/09/11 06:21:46 mwedel Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -374,7 +374,17 @@ int command_take (object *op, char *params)
 {
     object *tmp, *next;
 
-    tmp=(op->container) ? op->container->inv : op->below;
+    if (op->container) 
+	tmp=op->container;
+    else {
+	tmp=op->above;
+	if (tmp) while (tmp->above) {
+	    tmp=tmp->above;
+	}
+	if (!tmp)
+	    tmp=op->below;
+    }
+
     if (tmp==NULL) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Nothing to take!");
 	return 0;
@@ -604,20 +614,16 @@ void drop_object (object *op, object *tmp, long nrof)
  *  the player here.
  */
 #ifdef SAVE_INTERVAL
-    if (op->type == PLAYER)
-    {
-    if (!QUERY_FLAG(tmp, FLAG_UNPAID) &&
-      (tmp->nrof ? tmp->value * tmp->nrof : tmp->value > 2000))
-#if SAVE_INTERVAL
-      if((op->contr->last_save_time + SAVE_INTERVAL) <= time(NULL)) {
+    /* I'm not sure why there is a value check - since the save
+     * is done every SAVE_INTERVAL seconds, why care the value
+     * of what he is dropping?
+     */
+    if (op->type == PLAYER && !QUERY_FLAG(tmp, FLAG_UNPAID) &&
+      (tmp->nrof ? tmp->value * tmp->nrof : tmp->value > 2000) &&
+      (op->contr->last_save_time + SAVE_INTERVAL) <= time(NULL)) {
 	  save_player(op, 1);
 	  op->contr->last_save_time = time(NULL);
-     }
-#else
-	save_player(op,1); /* To avoid cheating */
-        };
-};
-#endif
+    }
 #endif /* SAVE_INTERVAL */
 
 
@@ -988,7 +994,8 @@ void examine_monster(object *op,object *tmp) {
 	new_draw_info(NDI_UNIQUE, 0,op,"It looks very ill.");
 }
 
-char *long_desc(object *tmp) {
+/* tmp is the object being described, pl is who is examing it. */
+char *long_desc(object *tmp, object *pl) {
   static char buf[VERY_BIG_BUF];
   char *cp;
 
@@ -1013,7 +1020,7 @@ char *long_desc(object *tmp) {
   case FOOD:
   case DRINK:
   case FLESH:
-    if(*(cp=describe_item(tmp))!='\0') {
+    if(*(cp=describe_item(tmp, pl))!='\0') {
 	int len;
 
 	strncpy(buf,query_name(tmp), VERY_BIG_BUF-1);
@@ -1044,24 +1051,9 @@ void examine(object *op, object *tmp) {
     if (tmp == NULL || tmp->type == CLOSE_CON)
 	return;
 
-    /* Eneq(csd.uu.se): If NO_PRETEXT is defined we should only print the name. */
-    if (QUERY_FLAG(tmp, FLAG_NO_PRETEXT)) {
-	strncpy(buf, long_desc(tmp), VERY_BIG_BUF-1);
-	buf[VERY_BIG_BUF-1]=0;
-    }
-    else {
-        /* Only quetzals can see the resistances on flesh. To realize
-	   this, we temporarily flag the flesh with SEE_INVISIBLE */
-        if (op->type == PLAYER && tmp->type == FLESH && is_dragon_pl(op))
-	    SET_FLAG(tmp, FLAG_SEE_INVISIBLE);
-	
-        strcpy(buf,"That is ");
-	
-	strncat(buf, long_desc(tmp), VERY_BIG_BUF-strlen(buf)-1);
-	buf[VERY_BIG_BUF-1]=0;
-	if (op->type == PLAYER && tmp->type == FLESH)
-	    CLEAR_FLAG(tmp, FLAG_SEE_INVISIBLE);
-    }
+    strcpy(buf,"That is ");
+    strncat(buf, long_desc(tmp, op), VERY_BIG_BUF-strlen(buf)-1);
+    buf[VERY_BIG_BUF-1]=0;
 
     new_draw_info(NDI_UNIQUE, 0,op,buf);
     buf[0]='\0';
@@ -1114,6 +1106,24 @@ void examine(object *op, object *tmp) {
 	}
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
     }
+    /* Where to wear this item */
+    for (i=0; i < NUM_BODY_LOCATIONS; i++) {
+	if (tmp->body_info[i]<-1) {
+	    if (op->body_info[i])
+		new_draw_info_format(NDI_UNIQUE, 0,op, 
+			"It goes %s (%d)", body_locations[i].use_name, -tmp->body_info[i]);
+	    else
+		new_draw_info_format(NDI_UNIQUE, 0,op, 
+		        "It goes %s", body_locations[i].nonuse_name);
+	} else if (tmp->body_info[i]) {
+	    if (op->body_info[i])
+		new_draw_info_format(NDI_UNIQUE, 0,op, 
+			"It goes %s", body_locations[i].use_name);
+	    else
+		new_draw_info_format(NDI_UNIQUE, 0,op, 
+			"It goes %s", body_locations[i].nonuse_name);
+	}
+    }
 
     if(tmp->weight) {
 	sprintf(buf,tmp->nrof>1?"They weigh %3.3f kg.":"It weighs %3.3f kg.",
@@ -1229,7 +1239,6 @@ int command_pickup (object *op, char *params)
       goto NEWPICKUP;
     }
     if(1)fprintf(stderr,"command_pickup: !params\n");
-    op->contr->count_left=0;
     set_pickup_mode(op, (op->contr->mode > 6)? 0: op->contr->mode+1);
     return 0;
   }
@@ -1311,6 +1320,13 @@ NEWPICKUP:
   sprintf(putstring,"%d POTION",i & PU_POTION?1:0);
   new_draw_info(NDI_UNIQUE, 0,op,putstring);
 
+  sprintf(putstring,"%d SPELLBOOK",i & PU_SPELLBOOK?1:0);
+  new_draw_info(NDI_UNIQUE, 0,op,putstring);
+  sprintf(putstring,"%d SKILLSCROLL",i & PU_SKILLSCROLL?1:0);
+  new_draw_info(NDI_UNIQUE, 0,op,putstring);
+  sprintf(putstring,"%d READABLES",i & PU_READABLES?1:0);
+  new_draw_info(NDI_UNIQUE, 0,op,putstring);
+
   new_draw_info(NDI_UNIQUE, 0,op,"");
 #endif
 
