@@ -1,11 +1,12 @@
 /*
  * static char *rcsid_c_wiz_c =
- *   "$Id: c_wiz.c,v 1.19 2002/01/16 10:51:32 jbontje Exp $";
+ *   "$Id: c_wiz.c,v 1.23 2002/09/07 22:08:07 mwedel Exp $";
  */
+
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copryight (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -22,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 /*
@@ -102,7 +103,6 @@ int command_kick (object *op, char *params)
 	op=pl->ob;
 	remove_ob(op);
 	op->direction=0;
-	op->contr->count_left=0;
 	new_draw_info_format(NDI_UNIQUE | NDI_ALL, 5, op,
 			     "%s is kicked out of the game.",op->name);
 	strcpy(op->contr->killer,"left");
@@ -207,80 +207,120 @@ int command_generate (object *op, char *params)
       return 1;
     }
 
+
+/* Enough of the DM functions seem to need this that I broke
+ * it out to a seperate function.  name is the person
+ * being saught, rq is who is looking for them.  This
+ * prints diagnostics messages, and returns the 
+ * other player, or NULL otherwise.
+ */
+static player *get_other_player_from_name(object *op, char *name)
+{
+    player *pl;
+
+    if (!name) return NULL;
+
+    for(pl=first_player;pl!=NULL;pl=pl->next) 
+	if(!strncmp(pl->ob->name, name, MAX_NAME)) 
+	    break;
+
+    if(pl==NULL) {
+	new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
+	return NULL;
+    }
+
+    if (pl->ob == op) {
+	new_draw_info(NDI_UNIQUE, 0, op, "You can't summon yourself next to yourself.");
+	return NULL;
+    }
+    if(pl->state != ST_PLAYING) {
+	new_draw_info(NDI_UNIQUE, 0,op,"That player can't be summoned right now.");
+	return NULL;
+    }
+    return pl;
+}
+
+int command_freeze(object *op, char *params)
+{
+    int ticks;
+    player *pl;
+
+    if (!params) {
+         new_draw_info(NDI_UNIQUE, 0,op,"Usage: freeze [ticks] <player>.");
+         return 1;
+    }
+
+    ticks = atoi(params);
+    if (ticks) {
+	while ((isdigit(*params) || isspace(*params)) && *params != 0) params++;
+	if (*params == 0) {
+	    new_draw_info(NDI_UNIQUE, 0,op,"Usage: freeze [ticks] <player>.");
+	    return 1;
+	}
+    } else ticks=100;
+    pl = get_other_player_from_name(op, params);
+    if (!pl) return 1;
+    new_draw_info(NDI_UNIQUE | NDI_RED, 0, pl->ob, "You have been frozen by the DM!");
+    new_draw_info_format(NDI_UNIQUE , 0, op,
+			 "You freeze %s for %d ticks", pl->ob->name, ticks);
+    pl->ob->speed_left = -(pl->ob->speed * ticks);
+    return 0;
+}
+
+
 int command_summon (object *op, char *params)
 {
-      int i;
-      object *dummy;
-  player *pl;
+    int i;
+    object *dummy;
+    player *pl;
 
-  if (!op)
-    return 0;
+    if (!op)
+	return 0;
 
-  if(params==NULL) {
+    if(params==NULL) {
          new_draw_info(NDI_UNIQUE, 0,op,"Usage: summon <player>.");
          return 1;
-      }
-      for(pl=first_player;pl!=NULL;pl=pl->next) 
-    if(!strncmp(pl->ob->name, params, MAX_NAME)) 
-          break;
-      if(pl==NULL) {
-        new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
-        return 1;
-      }
-      if (pl->ob == op) {
-        new_draw_info(NDI_UNIQUE, 0, op, "You can't summon yourself next to yourself.");
-        return 1;
-      }
-      if(pl->state != ST_PLAYING) {
-        new_draw_info(NDI_UNIQUE, 0,op,"That player can't be summoned right now.");
-        return 1;
-      }
-      i=find_free_spot(op->arch,op->map,op->x,op->y,1,8);
-      if (i==-1) {
+    }
+
+    pl = get_other_player_from_name(op, params);
+    if (!pl) return 1;
+
+    i=find_free_spot(op->arch,op->map,op->x,op->y,1,8);
+    if (i==-1) {
 	new_draw_info(NDI_UNIQUE, 0, op, "Can not find a free spot to place summoned player.");
 	return 1;
-      }
-      dummy=get_object();
-      EXIT_PATH(dummy)=add_string(op->map->path);
-      EXIT_X(dummy)=op->x+freearr_x[i];
-      EXIT_Y(dummy)=op->y+freearr_y[i];
-      enter_exit(pl->ob,dummy);
-      free_object(dummy);
-      new_draw_info(NDI_UNIQUE, 0,pl->ob,"You are summoned.");
-      new_draw_info(NDI_UNIQUE, 0,op,"OK.");
-      return 1;
     }
+    dummy=get_object();
+    EXIT_PATH(dummy)=add_string(op->map->path);
+    EXIT_X(dummy)=op->x+freearr_x[i];
+    EXIT_Y(dummy)=op->y+freearr_y[i];
+    enter_exit(pl->ob,dummy);
+    free_object(dummy);
+    new_draw_info(NDI_UNIQUE, 0,pl->ob,"You are summoned.");
+    new_draw_info(NDI_UNIQUE, 0,op,"OK.");
+
+    return 1;
+}
 
 /* Teleport next to target player */
 /* mids 01/16/2002 */
+
 int command_teleport (object *op, char *params) {
    int i;
    object *dummy;
    player *pl;
 
-   if (!op)
-      return 0;
+    if (!op)
+	return 0;
 
-   if (params==NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op,"Usage: teleport <player>.");
-      return 1;
-   }
+    if (params==NULL) {
+	new_draw_info(NDI_UNIQUE, 0,op,"Usage: teleport <player>.");
+	return 1;
+    }
+
+    pl = get_other_player_from_name(op, params);
+    if (!pl) return 1;
 
-   for (pl = first_player; pl != NULL; pl = pl->next) 
-      if (!strncmp(pl->ob->name, params, MAX_NAME)) 
-         break;
-   if (pl == NULL) {
-      new_draw_info(NDI_UNIQUE, 0, op, "No such player.");
-      return 1;
-   }
-   if (pl->ob == op) {
-      new_draw_info(NDI_UNIQUE, 0, op, "You can't teleport yourself next to yourself.");
-      return 1;
-   }
-   if (pl->state != ST_PLAYING) {
-      new_draw_info(NDI_UNIQUE, 0, op, "You can't teleport to that player right now.");
-      return 1;
-   }
    i = find_free_spot(pl->ob->arch, pl->ob->map, pl->ob->x, pl->ob->y, 1, 8);
    if (i==-1) {
       new_draw_info(NDI_UNIQUE, 0, op, "Can not find a free spot to teleport to.");
@@ -889,7 +929,7 @@ static int checkdm(object *op, char *pl_name, char *pl_passwd, char *pl_host)
   *pl_name=op->name?op->name:"*";
 #endif
 
-  sprintf (buf, "%s/%s", settings.datadir, DMFILE);
+  sprintf (buf, "%s/%s", settings.confdir, DMFILE);
   if ((dmfile = fopen(buf, "r")) == NULL) {
     LOG (llevDebug, "Could not find DM file.\n");
     return(0);
@@ -1006,7 +1046,7 @@ int command_loadplugin(object *op, char *params)
 {
     char buf[MAX_BUF];
 
-    strcpy(buf,DATADIR);
+    strcpy(buf,LIBDIR);
     strcat(buf,"/plugins/");
     strcat(buf,params);
     printf("Requested plugin file is %s\n", buf);
