@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_c =
- *   "$Id: commands.c,v 1.20 2002/03/20 06:27:46 mwedel Exp $";
+ *   "$Id: commands.c,v 1.24 2002/09/07 21:41:20 mwedel Exp $";
  */
 
 /*
@@ -59,7 +59,9 @@ CommArray_s Commands[] = {
   {"verify", command_malloc_verify,0.0},
 #endif
   {"apply", command_apply,	1.0},	/* should be variable */
+  {"applymode", command_applymode,	1.0},	/* should be variable */
   {"archs", command_archs,	0.0},
+  {"body", command_body,	0.0},
   {"brace", command_brace,	0.0},
   {"cast", command_cast,	0.2},	/* Is this right? */
   {"disarm", command_disarm,	1.0},
@@ -73,6 +75,7 @@ CommArray_s Commands[] = {
   {"hiscore", command_hiscore,	0.0},
   {"inventory", command_inventory,0.0},
   {"invoke", command_invoke,	1.0},
+  {"killpets", command_kill_pets,0.0},
   {"listen", command_listen,	0.0},
   {"logs", command_logs,	0.0},
   {"malloc", command_malloc,	0.0},
@@ -127,10 +130,10 @@ const int CommandsSize =sizeof(Commands) / sizeof(CommArray_s);
 
 CommArray_s CommunicationCommands [] = {
   /* begin emotions */
-  {"tell", command_tell,		0.0},
+  {"tell", command_tell,		0.1},
   {"reply", command_reply,		0.0},
-  {"say", command_say,			0.0},
-  {"shout", command_shout,		0.0},
+  {"say", command_say,			0.1},
+  {"shout", command_shout,		0.1},
   {"nod", command_nod,			0.0},
   {"dance", command_dance,		0.0},
   {"kiss", command_kiss,		0.0},
@@ -215,6 +218,7 @@ CommArray_s WizCommands [] = {
   {"dumpmap", command_dumpmap,0.0},
   {"forget_spell", command_forget_spell, 0.0},
   {"free", command_free,0.0},
+  {"freeze", command_freeze,0.0},
   {"goto", command_goto,0.0},
   {"invisible", command_invisible,0.0},
   {"kick", command_kick, 0.0},
@@ -262,8 +266,8 @@ const int Socket_CommandsSize =sizeof(Socket_Commands) / sizeof(CommArray_s);
  * to the various players/sockets.
  */
 CommArray_s Socket2_Commands[] = {
-  {"shout", command_shout,	0.0},
-  {"tell", command_tell,	0.0},
+  {"shout", command_shout,	0.1},
+  {"tell", command_tell,	0.1},
 };
 
 const int Socket2_CommandsSize =sizeof(Socket2_Commands) / sizeof(CommArray_s);
