@@ -1,11 +1,12 @@
 /*
  * static char *rcsid_gods_c =
- *   "$Id: gods.c,v 1.28 2001/10/30 02:30:20 michtoen Exp $";
+ *   "$Id: gods.c,v 1.31 2002/07/15 04:25:40 mwedel Exp $";
  */
+
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -22,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 
@@ -370,6 +371,12 @@ void become_follower (object *op, object *new_god) {
     check_special_prayers (op, new_god);
 }
 
+/* op is the player.
+ * exp_obj is the widsom experience.
+ * flag is the flag to check against.
+ * string is the string to print out.
+ */
+
 int worship_forbids_use (object *op, object *exp_obj, uint32 flag, char *string) {
 
   if(QUERY_FLAG(&op->arch->clone,flag))
@@ -402,13 +409,21 @@ void stop_using_item ( object *op, int type, int number ) {
  */
 
 void update_priest_flag (object *god, object *exp_ob, uint32 flag) {
-/* GROS - Corrected here the 'god flowers' bug. */
       if(QUERY_FLAG(god,flag)&&!QUERY_FLAG(exp_ob,flag))
           SET_FLAG(exp_ob,flag);
       else if(QUERY_FLAG(exp_ob,flag)&&!QUERY_FLAG(god,flag))
       {
-        if (!(QUERY_FLAG(&(exp_ob->arch->clone),flag)))
-                CLEAR_FLAG(exp_ob,flag);
+	/*  When this is called with the exp_ob set to the player,
+	 * this check is broken, because most all players arch
+	 * allow use of weapons.  I'm not actually sure why this
+	 * check is here - I guess if you had a case where the
+	 * value in the archetype (wisdom) should over ride the restrictions
+	 * the god places on it, this may make sense.  But I don't think
+	 * there is any case like that.
+	 */
+	 
+/*        if (!(QUERY_FLAG(&(exp_ob->arch->clone),flag)))*/
+	CLEAR_FLAG(exp_ob,flag);
       };
 }
 
@@ -1019,10 +1034,10 @@ int tailor_god_spell(object *spellop, object *caster) {
 	   free_string(spellop->title);
          spellop->title=add_string(god->name);
          if(spellop->title){
-	   char buf[MAX_BUF]; 
-	   sprintf(buf,"%s of %s",spellop->name,spellop->title);
-	   if (spellop->name) free_string(spellop->name);
-	   spellop->name=add_string(buf);
+	    char buf[MAX_BUF]; 
+	    sprintf(buf,"%s of %s",spellop->name,spellop->title);
+	    FREE_AND_COPY(spellop->name, buf);
+	    FREE_AND_COPY(spellop->name_pl, buf);
 	}
     } 
 
