@@ -1,12 +1,12 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.23 2002/03/03 20:05:58 avogl Exp $";
+ *   "$Id: init.c,v 1.26 2002/09/11 06:21:46 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -23,7 +23,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
 #include <global.h>
@@ -54,6 +54,7 @@ void set_dumpmon9() {settings.dumpvalues=9; }
 void set_dumpmont(char *name) {settings.dumpvalues=10; settings.dumparg=name; }
 void set_daemon() {settings.daemonmode=1; }
 void set_datadir(char *path) { settings.datadir=path; }
+void set_confdir(char *path) { settings.confdir=path; }
 void set_localdir(char *path) { settings.localdir=path; }
 void set_mapdir(char *path) { settings.mapdir=path; }
 void set_archetypes(char *path) { settings.archetypes=path; }
@@ -125,6 +126,7 @@ struct Command_Line_Options options[] = {
 {"-mon", 0, 1, set_mondebug},
 #ifndef SECURE
 {"-data",1,1, set_datadir},
+{"-conf",1,1, set_confdir},
 {"-local",1,1, set_localdir},
 {"-maps", 1, 1, set_mapdir},
 {"-arch", 1, 1, set_archetypes},
@@ -156,6 +158,7 @@ struct Command_Line_Options options[] = {
 {"-m8", 0, 3, set_dumpmon8},
 {"-m9", 0, 3, set_dumpmon9},
 {"-mt", 1, 3, set_dumpmont},
+{"-mexp", 0, 3, dump_experience},
 #endif
 {"-s", 0, 3, showscores},
 {"-score", 1, 3, showscoresparm},
@@ -225,7 +228,7 @@ static void load_settings()
     int	has_val,comp;
     FILE    *fp;
 
-    sprintf(buf,"%s/settings",settings.datadir);
+    sprintf(buf,"%s/settings",settings.confdir);
     /* We don't require a settings file at current time, but down the road,
      * there will probably be so many values that not having a settings file
      * will not be a good thing.
@@ -335,6 +338,16 @@ static void load_settings()
 		    "at least 0, %d is invalid\n", lev);
 	    else
 		settings.dynamiclevel = lev;
+	}
+	else if (!strcasecmp(buf,"simple_exp")) {
+	    if (!strcasecmp(cp,"on") || !strcasecmp(cp,"true")) {
+		settings.simple_exp=TRUE;
+	    } else if (!strcasecmp(cp,"off") || !strcasecmp(cp,"false")) {
+		settings.simple_exp=FALSE;
+	    } else {
+		LOG(llevError,"load_settings: Unkown value for simple_exp: %s\n",
+		    cp);
+	    }
 	} else {
 	    LOG(llevError,"Unknown value in settings file: %s\n", buf);
 	}
@@ -504,7 +517,7 @@ void init_startup() {
   int comp;
 
 #ifdef SHUTDOWN_FILE
-  sprintf(buf,"%s/%s",settings.datadir,SHUTDOWN_FILE);
+  sprintf(buf,"%s/%s",settings.confdir,SHUTDOWN_FILE);
   if ((fp = open_and_uncompress(buf, 0, &comp)) != NULL) {
     while (fgets(buf, MAX_BUF-1, fp) != NULL)
       printf("%s", buf);
@@ -558,15 +571,15 @@ void compile_info() {
   printf("Datadir:\t\t%s\n",settings.datadir);
   printf("Localdir:\t\t%s\n",settings.localdir);
 #ifdef PERM_FILE
-  printf("Perm file:\t<LIB>/%s\n",PERM_FILE);
+  printf("Perm file:\t<ETC>/%s\n",PERM_FILE);
 #endif
 #ifdef SHUTDOWN_FILE
-  printf("Shutdown file:\t<LIB>/%s\n",SHUTDOWN_FILE);
+  printf("Shutdown file:\t<ETC>/%s\n",SHUTDOWN_FILE);
 #endif
   printf("Save player:\t<true>\n");
   printf("Save mode:\t%4.4o\n",SAVE_MODE);
-  printf("Playerdir:\t<LIB>/%s\n",settings.playerdir);
-  printf("Itemsdir:\t<LIB>/%s\n", settings.uniquedir);
+  printf("Playerdir:\t<VAR>/%s\n",settings.playerdir);
+  printf("Itemsdir:\t<VAR>/%s\n", settings.uniquedir);
 #ifdef USE_CHECKSUM
   printf("Use checksum:\t<true>\n");
 #else
