@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skills_c =
- *   "$Id: skills.c,v 1.24 2002/02/06 05:54:47 mwedel Exp $";
+ *   "$Id: skills.c,v 1.29 2002/08/26 07:14:11 mwedel Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -106,6 +106,10 @@ int attempt_steal(object* op, object* who)
 	} else /* help npc to detect thief next time by raising its wisdom */ 
 	    op->stats.Wis += (op->stats.Int/5)+1;
     }
+    if (op->type == PLAYER && QUERY_FLAG(op, FLAG_WIZ)) {
+	new_draw_info(NDI_UNIQUE, 0, who, "You can't steal from the dungeon master!\n");
+	return 0;
+    }
 
     /* Ok then, go thru their inventory, stealing */
     for(tmp = op->inv; tmp != NULL; tmp = next) {
@@ -148,6 +152,12 @@ int attempt_steal(object* op, object* who)
 	}
     } /* for loop looking for an item */
 
+    if (!tmp) {
+	new_draw_info_format(NDI_UNIQUE, 0, who, "The %s has nothing you can steal!",
+			     query_name(op));
+	return 0;
+    }
+
     /* If you arent high enough level, you might get something BUT
      * the victim will notice your stealing attempt. Ditto if you
      * attempt to steal something heavy off them, they're bound to notice 
@@ -656,7 +666,7 @@ int do_skill_ident2(object *tmp,object *pl, int obj_class)
 		  identify(tmp);
      		  if (pl->type==PLAYER) {
         	    new_draw_info_format(NDI_UNIQUE, 0, pl,
-                      "You identify %s.", long_desc(tmp));
+                      "You identify %s.", long_desc(tmp, pl));
         	    if (tmp->msg) {
           		new_draw_info(NDI_UNIQUE, 0,pl, "The item has a story:");
           		new_draw_info(NDI_UNIQUE, 0,pl, tmp->msg);
@@ -933,7 +943,7 @@ int pray (object *pl) {
 	}
     }
 
-    new_draw_info(NDI_UNIQUE,0,pl,buf);
+    new_draw_info(NDI_BLACK,0,pl,buf);
      
     if(pl->stats.grace < pl->stats.maxgrace) {
 	pl->stats.grace++;
@@ -1099,6 +1109,10 @@ int write_note(object *pl, object *item, char *msg) {
 	  skills[SK_INSCRIPTION].name);
 	return 0;
   }
+  if(strstr(msg,"endmsg")) {
+  	new_draw_info(NDI_UNIQUE,0,pl,"Trying to cheat now are we?");
+	return 0;
+  }
 #ifdef PLUGINS
   /* GROS: Handle for plugin book writing (trigger) event */
   if(item->event_hook[EVENT_TRIGGER] != NULL)
@@ -1426,14 +1440,12 @@ void do_throw(object *op, object *toss_item, int dir) {
     if(throw_ob==NULL) {
 	if(op->type==PLAYER) {
 	    new_draw_info(NDI_UNIQUE, 0,op,"You have nothing to throw.");
-	    op->contr->count_left=0; /* ?? is this needed?? */
 	}
 	return;
     }
     if (QUERY_FLAG(throw_ob, FLAG_STARTEQUIP)) {
 	if (op->type==PLAYER) {
 	    new_draw_info(NDI_UNIQUE, 0, op, "The gods won't let you throw that.");
-	    op->contr->count_left=0; /* ?? is this needed?? */
 	}
 	return;
     }
@@ -1500,7 +1512,6 @@ void do_throw(object *op, object *toss_item, int dir) {
 	    }
 	    else
 		new_draw_info(NDI_UNIQUE, 0,op,"Something is in the way.");
-	    op->contr->count_left=0; /* needed?? */
 	}
 	return;
     } /* if object can't be thrown */
