@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_util_c =
- *   "$Id: spell_util.c,v 1.63 2001/12/23 08:09:05 darth_bob Exp $";
+ *   "$Id: spell_util.c,v 1.66 2002/09/07 22:08:07 mwedel Exp $";
  */
 
 
@@ -37,11 +37,6 @@
 #endif
 #include <sounds.h>
 
-char *range_name[range_size] = {
-  "none", "bow", "magic", "wand", "rod", "scroll", "horn"
-	,"steal"
-};
-
 archetype *spellarch[NROFREALSPELLS];
 
 void init_spells() {
@@ -181,25 +176,19 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
 	 if( spells[type].onself) dir = 0;
   }
 
-  if(!(QUERY_FLAG(op, FLAG_WIZ))&&
-	  (op->type==PLAYER)&&
-	  (op->contr->shoottype==range_magic)&&
-     (item!=spellPotion)&& 
-     (!(IS_SUMMON_SPELL(type)&&op->contr->golem!=NULL)))
-  {
-	if(!spells[type].cleric&& op->stats.sp<SP_level_spellpoint_cost(op,caster,type))
-	{
-	  new_draw_info(NDI_UNIQUE, 0,op,"You don't have enough mana.");
-	  op->contr->count_left=0;
-	  return 0;
+  if(!(QUERY_FLAG(op, FLAG_WIZ))&& (op->type==PLAYER) && (op->contr->shoottype==range_magic) &&
+     (item!=spellPotion)&&  (!(IS_SUMMON_SPELL(type)&&op->contr->golem!=NULL))) {
+	if( !spells[type].cleric && op->stats.sp<SP_level_spellpoint_cost(op,caster,type)) {
+		new_draw_info(NDI_UNIQUE, 0,op,"You don't have enough mana.");
+		return 0;
 	}
-	if(spells[type].cleric&&op->stats.grace<SP_level_spellpoint_cost(op,caster,type))
+	else if(spells[type].cleric && op->stats.grace<SP_level_spellpoint_cost(op,caster,type))
 	{
 	  /* it's possible for grace to go negative */
 	  /* Fine - let grace go negative, but how negative it is should really
 	   * put a limit on things - in the old method, chance was the same
 	   * no matter how negative it was.
-	   *Instead of subtracting 10 from the roll, add in grace (which is
+	   * Instead of subtracting 10 from the roll, add in grace (which is
 	   * negative).  This puts a real limit on things.
 	   */
 	  if(random_roll(0, op->stats.Wis-1, op, PREFER_HIGH) + op->stats.grace -
@@ -211,7 +200,6 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
 	  {
 		  prayer_failure(op,op->stats.grace,SP_level_spellpoint_cost(op,caster,type));
 			  new_draw_info_format(NDI_UNIQUE, 0,op,"%s ignores your prayer.",godname);
-		  op->contr->count_left=0;
 			  return 0;
 	  }
 	}
@@ -219,7 +207,6 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
 
   if (caster->path_denied & s->path) {
     new_draw_info(NDI_UNIQUE, 0,op, "You are unable to cast that spell.");
-    if(op->type==PLAYER) op->contr->count_left=0;
     return 0;
   }
   /* If it is an ability, assume that the designer of the archetype knows
@@ -227,7 +214,6 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
    */
   if (item==spellNormal && !ability && SK_level(caster) < s->level && !QUERY_FLAG(op,FLAG_WIZ)) {
     new_draw_info(NDI_UNIQUE, 0,op, "You lack enough skill to cast that spell.");
-    if(op->type==PLAYER) op->contr->count_left=0;
     return 0;
   }
 
@@ -270,16 +256,10 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
     case range_magic:
       new_draw_info(NDI_UNIQUE, 0,op,"Something blocks your spellcasting.");
       break;
-    case range_wand:
-      new_draw_info(NDI_UNIQUE, 0,op,"Something blocks the magic of your wand.");
+    case range_misc:
+      new_draw_info(NDI_UNIQUE, 0,op,"Something blocks the magic of your item.");
       break;
-    case range_rod:
-      new_draw_info(NDI_UNIQUE, 0,op,"Something blocks the magic of your rod.");
-      break;
-    case range_horn:
-      new_draw_info(NDI_UNIQUE, 0,op,"Something blocks the magic of your horn.");
-      break;
-    case range_scroll:
+    case range_golem:
       new_draw_info(NDI_UNIQUE, 0,op,"Something blocks the magic of your scroll.");
       break;
     default:
@@ -684,29 +664,10 @@ if (item == spellNormal && !ability ){
     }
     break;
   case SP_RUNE_MAGIC:
-    { int total_sp_cost, spellinrune;
-      spellinrune=look_up_spell_by_name(op,stringarg);
-      if(spellinrune!=-1) {
-      total_sp_cost=SP_level_spellpoint_cost(op,caster,spellinrune)
-			+spells[spellinrune].sp;
-	if(op->stats.sp<total_sp_cost) {
-	  new_draw_info(NDI_UNIQUE, 0,op,"Not enough spellpoints.");
-#ifdef CASTING_TIME
-	  /* free the spell arg */
-	  if(stringarg) {free(stringarg);stringarg=NULL; };
-#endif
-	  return 0;
-	}
-	success=write_rune(op,dir,spellinrune,caster->level,stringarg);
-	return (success ? total_sp_cost : 0);
-      }
-#ifdef CASTING_TIME
-	  /* free the spell arg */
-	  if(stringarg) {free(stringarg);stringarg=NULL; };
-#endif
-      return 0;
-    }
+  case SP_GLYPH:
+    return cast_generic_rune(op, caster, dir, stringarg, type);
     break;
+
   case SP_RUNE_MARK:
     if(caster->type == PLAYER)
       success=write_rune(op,dir,0,-2,stringarg);
@@ -830,8 +791,6 @@ int summon_monster(object *op,object *caster,int dir,archetype *at,int spellnum)
     dir=find_free_spot(NULL,op->map,op->x,op->y,1,9);
   if((dir==-1) || blocked(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir])) {
     new_draw_info(NDI_UNIQUE, 0,op,"There is something in the way.");
-    if(op->type==PLAYER)
-      op->contr->count_left=0;
     return 0;
   }
   tmp=arch_to_object(at);
@@ -845,7 +804,7 @@ int summon_monster(object *op,object *caster,int dir,archetype *at,int spellnum)
     set_owner(tmp,op);
     op->contr->golem=tmp;
     /* give the player control of the golem */
-    op->contr->shoottype=range_scroll;
+    op->contr->shoottype=range_golem;
   } else {
     if(QUERY_FLAG(op, FLAG_FRIENDLY)) {
       object *owner = get_owner(op);
@@ -2405,7 +2364,7 @@ void init_spell_param()
   int spellindex;
   int spl;  /*  the spellpoint level dependency */
 
-  sprintf(fname,"%s/spell_params",settings.datadir);
+  sprintf(fname,"%s/spell_params",settings.confdir);
   if(! (spell_params=fopen(fname,"r")))
     {
 	perror(fname);
