@@ -1,7 +1,6 @@
-
 /*
  * static char *rcsid_init_c =
- *    "$Id: init.c,v 1.23 2001/12/31 08:36:04 mwedel Exp $";
+ *    "$Id: init.c,v 1.27 2002/08/26 07:14:11 mwedel Exp $";
  */
 
 /*
@@ -75,7 +74,7 @@ void InitConnection(NewSocket *ns, uint32 from)
 	if(ioctlsocket(ns->fd, FIONBIO , &temp) == -1)
 		LOG(llevError,"InitConnection:  Error on ioctlsocket.\n");
 #else 
-    if (fcntl(ns->fd, F_SETFL, O_NDELAY)==-1) {
+    if (fcntl(ns->fd, F_SETFL, O_NONBLOCK)==-1) {
 		LOG(llevError,"InitConnection:  Error on fcntl.\n");
     }
 #endif /* end win32 */
@@ -103,8 +102,7 @@ void InitConnection(NewSocket *ns, uint32 from)
     ns->ext2 = 0;
     ns->ext_title_flag = 1;
     ns->skillexp = 0;
-    ns->map1cmd = 0;
-    ns->map2cmd = 0;
+    ns->mapmode = Map0Cmd;
     ns->darkness = 1;
     ns->newanim = 0;
     ns->status = Ns_Add;
@@ -113,6 +111,7 @@ void InitConnection(NewSocket *ns, uint32 from)
     ns->mapx = 11;
     ns->mapy = 11;
     ns->newmapcmd= 0;
+    ns->itemcmd = 1;	/* Default is version item1 command*/
 
     /* we should really do some checking here - if total clients overflows
      * we need to do something more intelligent, because client id's will start
@@ -230,9 +229,9 @@ void init_ericserver()
         defined(__sun__) || defined(linux) || defined(SVR4) || defined(__FreeBSD__) || \
 	defined(__OpenBSD__) || defined(WIN32) /* ---win32 add this here */
     {
-	char tmp =1;
+	int tmp =1;
 
-	if(setsockopt(init_sockets[0].fd,SOL_SOCKET,SO_REUSEADDR, &tmp, sizeof(&tmp))) {
+	if(setsockopt(init_sockets[0].fd,SOL_SOCKET,SO_REUSEADDR, &tmp, sizeof(tmp))) {
 	    perror("error on setsockopt REUSEADDR");
 	    LOG(llevError,"error on setsockopt REUSEADDR\n");
 	}
@@ -305,8 +304,6 @@ void free_newsocket(NewSocket *ns)
     }
     if (ns->stats.range)
 	free(ns->stats.range);
-    if (ns->stats.ext_title)
-        free(ns->stats.ext_title);
     if (ns->stats.title)
         free(ns->stats.title);
     if (ns->comment)
