@@ -1,13 +1,13 @@
 
 /*
  * static char *rcsid_item_c =
- *    "$Id: item.c,v 1.21 2001/12/23 08:17:36 mwedel Exp $";
+ *    "$Id: item.c,v 1.24 2002/06/01 05:38:07 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2000 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
 /* This containes item logic for client/server.  IT doesn't contain
@@ -135,9 +135,9 @@ unsigned int query_flags (object *op)
 void esrv_draw_look(object *pl)
 {
     object *tmp, *last;
-    int flags, got_one=0,anim_speed, start_look=0, end_look=0;
+    int flags, got_one=0,anim_speed, start_look=0, end_look=0, len;
     SockList sl;
-    char buf[MAX_BUF];
+    char buf[MAX_BUF], *item_p,item_n[MAX_BUF];
 
     if (!pl->contr->socket.update_look) {
 	LOG(llevDebug,"esrv_draw_look called when update_look was not set\n");
@@ -156,7 +156,7 @@ void esrv_draw_look(object *pl)
     sl.buf=malloc(MAXSOCKBUF);
 
     Write_String_To_Socket(&pl->contr->socket, "delinv 0", strlen("delinv 0"));
-    strcpy((char*)sl.buf,"item1 ");
+    sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
     sl.len=strlen((char*)sl.buf);
 
     SockList_AddInt(&sl, 0);
@@ -174,6 +174,8 @@ void esrv_draw_look(object *pl)
 	SockList_AddShort(&sl,0);
 	SockList_AddChar(&sl, 0);
 	SockList_AddInt(&sl, 0);
+	if (pl->contr->socket.itemcmd == 2)
+	    SockList_AddShort(&sl, 0);
     }
 
     for (last=NULL; tmp!=last; tmp=tmp->below) {
@@ -186,6 +188,9 @@ void esrv_draw_look(object *pl)
 	    if (++start_look < pl->contr->socket.look_position) continue;
 	    end_look++;
 	    if (end_look > NUM_LOOK_OBJECTS) {
+		/* What we basically do is make a 'fake' object - when the user applies it,
+		 * we notice the special tag the object has, and act accordingly.
+		 */
 		SockList_AddInt(&sl, 0x80000000 | (pl->contr->socket.look_position+ NUM_LOOK_OBJECTS));
 		SockList_AddInt(&sl, 0);
 		SockList_AddInt(&sl, -1);
@@ -195,6 +200,8 @@ void esrv_draw_look(object *pl)
 		SockList_AddShort(&sl,0);
 		SockList_AddChar(&sl, 0);
 		SockList_AddInt(&sl, 0);
+		if (pl->contr->socket.itemcmd == 2)
+		    SockList_AddShort(&sl, 0);
 		break;
 	    }
 
@@ -212,22 +219,17 @@ void esrv_draw_look(object *pl)
 	    SockList_AddInt(&sl, flags);
 	    SockList_AddInt(&sl, QUERY_FLAG(tmp, FLAG_NO_PICK) ? -1 : WEIGHT(tmp));
 	    SockList_AddInt(&sl, tmp->face->number);
-        if (pl->contr->socket.sc_version>=1024) {
-		int len;
-		char *item_p,item_n[MAX_BUF];
 
-		strncpy(item_n,query_base_name(tmp, 0),127);
-		item_n[127]=0;
-		len=strlen(item_n);
-		item_p=query_base_name(tmp, 1);
-		strncpy(item_n+len+1, item_p, 127);
-		item_n[254]=0;
-		len += strlen(item_n+1+len) + 1;
-		SockList_AddChar(&sl, (char ) len);
-		memcpy(sl.buf+sl.len, item_n, len);
-		sl.len += len;
-	    } else
-		add_stringlen_to_sockbuf(query_base_name(tmp,0), &sl);
+	    strncpy(item_n,query_base_name(tmp, 0),127);
+	    item_n[127]=0;
+	    len=strlen(item_n);
+	    item_p=query_base_name(tmp, 1);
+	    strncpy(item_n+len+1, item_p, 127);
+	    item_n[254]=0;
+	    len += strlen(item_n+1+len) + 1;
+	    SockList_AddChar(&sl, (char ) len);
+	    memcpy(sl.buf+sl.len, item_n, len);
+	    sl.len += len;
 
 	    SockList_AddShort(&sl,tmp->animation_id);
 	    anim_speed=0;
@@ -242,12 +244,16 @@ void esrv_draw_look(object *pl)
 	    }
 	    SockList_AddChar(&sl, (char) anim_speed);
 	    SockList_AddInt(&sl, tmp->nrof);
+
+	    if (pl->contr->socket.itemcmd == 2)
+		SockList_AddShort(&sl, tmp->client_type);
+
 	    SET_FLAG(tmp, FLAG_CLIENT_SENT);
 	    got_one++;
 
 	    if (sl.len > (MAXSOCKBUF-MAXITEMLEN)) {
 		Send_With_Handling(&pl->contr->socket, &sl);
-		strcpy((char*)sl.buf,"item1 ");
+		sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
 		sl.len=strlen((char*)sl.buf);
 		SockList_AddInt(&sl, 0);
 		got_one=0;
@@ -263,9 +269,9 @@ void esrv_draw_look(object *pl)
 void esrv_send_inventory(object *pl, object *op)
 {
     object *tmp;
-    int flags, got_one=0, anim_speed;
+    int flags, got_one=0, anim_speed, len;
     SockList sl;
-    char item_n[MAX_BUF];
+    char item_n[MAX_BUF], *item_p;
     
     sl.buf=malloc(MAXSOCKBUF);
 
@@ -273,7 +279,7 @@ void esrv_send_inventory(object *pl, object *op)
     sl.len=strlen((char*)sl.buf);
     Send_With_Handling(&pl->contr->socket, &sl);
 
-    strcpy((char*)sl.buf,"item1 ");
+    sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
     sl.len=strlen((char*)sl.buf);
 
     SockList_AddInt(&sl, op->count);
@@ -293,22 +299,16 @@ void esrv_send_inventory(object *pl, object *op)
 	    SockList_AddInt(&sl, QUERY_FLAG(tmp, FLAG_NO_PICK) ? -1 : WEIGHT(tmp));
 	    SockList_AddInt(&sl, tmp->face->number);
         
-	    if (pl->contr->socket.sc_version>=1024) {
-		int len;
-		char *item_p;
-
-		strncpy(item_n,query_base_name(tmp, 0),127);
-		item_n[127]=0;
-		len=strlen(item_n);
-		item_p=query_base_name(tmp, 1);
-		strncpy(item_n+len+1, item_p, 127);
-		item_n[254]=0;
-		len += strlen(item_n+1+len) + 1;
-		SockList_AddChar(&sl, (char) len);
-		memcpy(sl.buf+sl.len, item_n, len);
-		sl.len += len;
-	    } else
-		add_stringlen_to_sockbuf(query_base_name(tmp,0), &sl);
+	    strncpy(item_n,query_base_name(tmp, 0),127);
+	    item_n[127]=0;
+	    len=strlen(item_n);
+	    item_p=query_base_name(tmp, 1);
+	    strncpy(item_n+len+1, item_p, 127);
+	    item_n[254]=0;
+	    len += strlen(item_n+1+len) + 1;
+	    SockList_AddChar(&sl, (char) len);
+	    memcpy(sl.buf+sl.len, item_n, len);
+	    sl.len += len;
 
 	    SockList_AddShort(&sl,tmp->animation_id);
 	    anim_speed=0;
@@ -323,16 +323,18 @@ void esrv_send_inventory(object *pl, object *op)
 	    }
 	    SockList_AddChar(&sl, (char)anim_speed);
 	    SockList_AddInt(&sl, tmp->nrof);
+	    if (pl->contr->socket.itemcmd == 2)
+		SockList_AddShort(&sl, tmp->client_type);
 	    SET_FLAG(tmp, FLAG_CLIENT_SENT);
 	    got_one++;
 
 	    /* IT is possible for players to accumulate a huge amount of
 	     * items (especially with some of the bags out there) to
-	     * overflow the buffer.  IF so, send multiple item1 commands.
+	     * overflow the buffer.  IF so, send multiple item commands.
 	     */
 	    if (sl.len > (MAXSOCKBUF-MAXITEMLEN)) {
 		Send_With_Handling(&pl->contr->socket, &sl);
-		strcpy((char*)sl.buf,"item1 ");
+		sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
 		sl.len=strlen((char*)sl.buf);
 		SockList_AddInt(&sl, op->count);
 		got_one=0;
@@ -392,22 +394,19 @@ void esrv_update_item(int flags, object *pl, object *op)
 	SockList_AddInt(&sl, op->face->number);
     }
     if (flags & UPD_NAME) {
-	if (pl->contr->socket.sc_version>=1024) {
-	    int len;
-	    char *item_p, item_n[MAX_BUF];
+	int len;
+	char *item_p, item_n[MAX_BUF];
 
-	    strncpy(item_n,query_base_name(op, 0),127);
-	    item_n[127]=0;
-	    len=strlen(item_n);
-	    item_p=query_base_name(op, 1);
-	    strncpy(item_n+len+1, item_p, 127);
-	    item_n[254]=0;
-	    len += strlen(item_n+1+len) + 1;
-	    SockList_AddChar(&sl, (char)len);
-	    memcpy(sl.buf+sl.len, item_n, len);
-	    sl.len += len;
-	} else
-	    add_stringlen_to_sockbuf(query_base_name(op,0), &sl);
+	strncpy(item_n,query_base_name(op, 0),127);
+	item_n[127]=0;
+	len=strlen(item_n);
+	item_p=query_base_name(op, 1);
+	strncpy(item_n+len+1, item_p, 127);
+	item_n[254]=0;
+	len += strlen(item_n+1+len) + 1;
+	SockList_AddChar(&sl, (char)len);
+	memcpy(sl.buf+sl.len, item_n, len);
+	sl.len += len;
     }
     if (flags & UPD_ANIM) 
 	    SockList_AddShort(&sl,op->animation_id);
@@ -434,9 +433,9 @@ void esrv_update_item(int flags, object *pl, object *op)
 
 void esrv_send_item(object *pl, object*op)
 {
-    int anim_speed;
+    int anim_speed, len;
     SockList sl;
-    char item_n[MAX_BUF];
+    char item_n[MAX_BUF], *item_p;
     
     /* If this is not the player object, do some more checks */
     if (op!=pl) {
@@ -454,7 +453,7 @@ void esrv_send_item(object *pl, object*op)
 
     sl.buf=malloc(MAXSOCKBUF);
 
-    strcpy((char*)sl.buf,"item1 ");
+    sprintf((char*)sl.buf,"item%d ", pl->contr->socket.itemcmd);
     sl.len=strlen((char*)sl.buf);
 
     SockList_AddInt(&sl, op->env? op->env->count:0);
@@ -470,22 +469,16 @@ void esrv_send_item(object *pl, object*op)
     SockList_AddInt(&sl, WEIGHT(op));
     SockList_AddInt(&sl, op->face->number);
     
-    if (pl->contr->socket.sc_version>=1024) {
-	int len;
-	char *item_p;
-
-	strncpy(item_n,query_base_name(op, 0),127);
-	item_n[127]=0;
-	len=strlen(item_n);
-	item_p=query_base_name(op, 1);
-	strncpy(item_n+len+1, item_p, 127);
-	item_n[254]=0;
-	len += strlen(item_n+1+len) + 1;
-	SockList_AddChar(&sl, (char)len);
-	memcpy(sl.buf+sl.len, item_n, len);
-	sl.len += len;
-    } else
-	add_stringlen_to_sockbuf(query_base_name(op,0), &sl);
+    strncpy(item_n,query_base_name(op, 0),127);
+    item_n[127]=0;
+    len=strlen(item_n);
+    item_p=query_base_name(op, 1);
+    strncpy(item_n+len+1, item_p, 127);
+    item_n[254]=0;
+    len += strlen(item_n+1+len) + 1;
+    SockList_AddChar(&sl, (char)len);
+    memcpy(sl.buf+sl.len, item_n, len);
+    sl.len += len;
 
     SockList_AddShort(&sl,op->animation_id);
     anim_speed=0;
@@ -500,6 +493,9 @@ void esrv_send_item(object *pl, object*op)
     }
     SockList_AddChar(&sl, (char)anim_speed);
     SockList_AddInt(&sl, op->nrof);
+    if (pl->contr->socket.itemcmd == 2)
+	SockList_AddShort(&sl, op->client_type);
+
     Send_With_Handling(&pl->contr->socket, &sl);
     SET_FLAG(op, FLAG_CLIENT_SENT);
     free(sl.buf);
@@ -772,7 +768,14 @@ void esrv_move_object (object *pl, tag_t to, tag_t tag, long nrof)
 #if 0
     printf ("Sacks name was '%s'.\n", env->name);
 #endif
-    put_object_in_sack (pl, env, op, nrof);
+    /* put_object_in_sack presumes that necessary sanity checking
+     * has already been done (eg, it can be picked up and fits in
+     * in a sack, so check for those things.  We should also check
+     * an make sure env is in fact a container for that matter.
+     */
+    if (env->type == CONTAINER && can_pick(pl, op) && sack_can_hold(pl, env, op, nrof)) {
+	put_object_in_sack (pl, env, op, nrof);
+    }
 }
 
 
