@@ -1,13 +1,13 @@
 
 /*
  * static char *rcsid_loop_c =
- *    "$Id: loop.c,v 1.17 2001/12/18 08:58:03 garbled Exp $";
+ *    "$Id: loop.c,v 1.19 2002/07/25 06:57:14 mwedel Exp $";
  */
 
 /*
     CrossFire, A Multiplayer game for X-windows
 
-    Copyright (C) 2001 Mark Wedel
+    Copyright (C) 2002 Mark Wedel & The Crossfire Development Team
     Copyright (C) 1992 Frank Tore Johansen
 
     This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    The author can be reached via e-mail to mwedel@scruz.net
+    The author can be reached via e-mail to crossfire-devel@real-time.com
 */
 
 /* socket.c mainly deals with initialization and higher level socket
@@ -81,6 +81,7 @@ typedef void (*func_uint8_int_pl)(char*, int, player *);
 struct PlCmdMapping {
     char *cmdname;
     func_uint8_int_pl cmdproc;
+    uint8   flag;
 };
 
 /*
@@ -91,30 +92,65 @@ struct PlCmdMapping {
  * unsigned (pretty much needs to be for binary data), however, most
  * of these treat it only as strings, so it makes things easier
  * to cast it here instead of a bunch of times in the function itself.
+ * flag is 1 if the player must be in the playing state to issue the
+ * command, 0 if they can issue it at any time.
  */
 static struct PlCmdMapping plcommands[] = {
-    { "examine",	ExamineCmd },
-    { "apply",		ApplyCmd },
-    { "move",		MoveCmd },
-    { "reply",		ReplyCmd},
-    { "command",	PlayerCmd},
-    { "ncom",		(func_uint8_int_pl)NewPlayerCmd},
-    { "lookat",		LookAt},
-    { "askface",	SendFaceCmd},	/* Added: phil */
-    { "mapredraw",	MapRedrawCmd},	/* Added: phil */
-    { "lock",		(func_uint8_int_pl)LockItem},
-    { "mark",		(func_uint8_int_pl)MarkItem},
-    { NULL, NULL}	/* terminator */
-};
-static struct NsCmdMapping nscommands[] = {
-    { "version",	VersionCmd },
-    { "addme",		AddMeCmd },
-    { "setfacemode",	SetFaceMode},
-    { "setsound",	SetSound},
-    { "setup",		SetUp},
+    { "examine",	ExamineCmd,	1},
+    { "apply",		ApplyCmd,	1},
+    { "move",		MoveCmd,	1},
+    { "reply",		ReplyCmd,	0},
+    { "command",	PlayerCmd,	1},
+    { "ncom",		(func_uint8_int_pl)NewPlayerCmd, 1},
+    { "lookat",		LookAt,		1},
+    { "lock",		(func_uint8_int_pl)LockItem,	1},
+    { "mark",		(func_uint8_int_pl)MarkItem,	1},
+    { "mapredraw",	MapRedrawCmd,	0},	/* Added: phil */
     { NULL, NULL}	/* terminator */
 };
 
+static struct NsCmdMapping nscommands[] = {
+    { "addme",		AddMeCmd },
+    { "askface",	SendFaceCmd},	/* Added: phil */
+    { "requestinfo",	RequestInfo},
+    { "setfacemode",	SetFaceMode},
+    { "setsound",	SetSound},
+    { "setup",		SetUp},
+    { "version",	VersionCmd },
+    { NULL, NULL}	/* terminator */
+};
+
+/* RequestInfo is sort of a meta command - there is some specific
+ * request of information, but we call other functions to provide
+ * that information.
+ */
+void RequestInfo(char *buf, int len, NewSocket *ns)
+{
+    char    *params=NULL, *cp;
+    /* No match */
+    char bigbuf[MAX_BUF];
+    int slen;
+
+    /* Set up replyinfo before we modify any of the buffers - this is used
+     * if we don't find a match.
+     */
+    strcpy(bigbuf,"replyinfo ");
+    slen = strlen(bigbuf);
+    safe_strcat(bigbuf, buf, &slen, MAX_BUF);
+
+    /* find the first space, make it null, and update the
+     * params pointer.
+     */
+    for (cp = buf; *cp != '\0'; cp++)
+	if (*cp==' ') {
+	    *cp = '\0';
+	    params = cp + 1;
+	    break;
+	}
+    if (!strcmp(buf, "image_info")) send_image_info(ns, params);
+    else if (!strcmp(buf,"image_sums")) send_image_sums(ns, params);
+    else Write_String_To_Socket(ns, bigbuf, len);
+}
 
 void Handle_Oldsocket(NewSocket *ns)
 {
@@ -312,10 +348,17 @@ void HandleClient(NewSocket *ns, player *pl)
 		return;
 	    }
 	}
-	/* Only valid players can use these commands */
+	/* Player must be in the playing state or the flag on the
+	 * the command must be zero for the user to use the command -
+	 * otherwise, a player cam save, be in the play_again state, and
+	 * the map they were on getsswapped out, yet things that try to look
+	 * at the map causes a crash.  If the command is valid, but
+	 * one they can't use, we still swallow it up.
+	 */
 	if (pl) for (i=0; plcommands[i].cmdname !=NULL; i++) {
 	    if (strcmp((char*)ns->inbuf.buf+2,plcommands[i].cmdname)==0) {
-		plcommands[i].cmdproc((char*)data,len,pl);
+		if (pl->state == ST_PLAYING || plcommands[i].flag == 0)
+		    plcommands[i].cmdproc((char*)data,len,pl);
 		ns->inbuf.len=0;
 		return;
 	    }
